local _G = getfenv(0)

local donothing = function() end

local frames = {} -- Stores globally created frames, and their internal properties.

local FrameClass = {} -- A class for creating frames.
FrameClass.methods = { "SetScript", "RegisterEvent", "UnregisterEvent", "UnregisterAllEvents", "Show", "Hide", "IsShown", "ClearAllPoints", "SetParent" }
function FrameClass:New()
	local frame = {}
	for i,method in ipairs(self.methods) do
		frame[method] = self[method]
	end
	local frameProps = {
		events = {},
		scripts = {},
		timer = GetTime(),
		isShow = true,
		parent = nil,
	}
	return frame, frameProps
end
function FrameClass:SetScript(script,handler)
	frames[self].scripts[script] = handler
end
function FrameClass:RegisterEvent(event)
	frames[self].events[event] = true
end
function FrameClass:UnregisterEvent(event)
	frames[self].events[event] = nil
end
function FrameClass:UnregisterAllEvents(frame)
	for event in pairs(frames[self].events) do
		frames[self].events[event] = nil
	end
end
function FrameClass:Show()
	frames[self].isShow = true
end
function FrameClass:Hide()
	frames[self].isShow = false
end
function FrameClass:IsShown()
	return frames[self].isShow
end
function FrameClass:ClearAllPoints()
end
function FrameClass:SetParent(parent)
	frames[self].parent = parent
end


function CreateFrame(kind, name, parent)
	local frame,internal = FrameClass:New()
	internal.parent = parent
	frames[frame] = internal
	if name then
		_G[name] = frame
	end
	return frame
end

local _cvars = {};
function GetCVar(name)
  return _cvars[name];
end

function SetCVar(name,value)
  _cvars[name] = value;
end

function UnitName(unit)
  return GetCVar("UnitName_"..unit);
end

function UnitOnTaxi(unit)
  return nil;
end

function GetRealmName()
  return "Realm Name"
end

function UnitClass(unit)
  return GetCVar("UnitClass1_"..unit),GetCVar("UnitClass2_"..unit);
end

function UnitHealthMax()
	return 100
end

function UnitHealth()
	return 50
end

function GetNumRaidMembers()
	return 0
end

function GetNumPartyMembers()
	return 0
end

function UnitIsPartyLeader(unit)
  return nil;
end

function UnitLevel(unit)
  return 70;
end

function UnitIsConnected(unit)
  return 1;
end

function UnitIsDeadOrGhost(unit)
  return nil;
end

function UnitIsCharmed(unit)
  return nil;
end

function GetRealZoneText()
  return GetCVar("RealZoneText");
end

function IsInInstance()
  return nil;
end

function IsGuildLeader()
  return nil;
end

function CanViewOfficerNote()
  return nil;
end

function CanEditOfficerNote()
  return nil;
end

function GuildRoster()
  WoWAPI_FireEvent("GUILD_ROSTER_UPDATE");
end

function GetNumGuildMembers(offline)
  return GetCVar("NumGuildMembersOnline");
end

function GetGuildRosterInfo(index)
  local name = GetCVar("GuildMember_"..index);
  local clas = GetCVar("UnitClass1_"..name);
  return GetCVar("UnitName_"..name), "Member", "3", 70, clas, GetCVar("RealZoneText"), name.." note", name.." off note", 1, "";
end

FACTION_HORDE = "Horde"
FACTION_ALLIANCE = "Alliance"

function UnitFactionGroup(unit)
	return "Horde", "Horde"
end

function UnitRace(unit)
	return "Undead", "Scourge"
end


_time = 0
function GetTime()
	return _time
end

function IsAddOnLoaded() return nil end

SlashCmdList = {}

function __WOW_Input(text)
	local a,b = string.find(text, "^/%w+")
	local arg, text = string.sub(text, a,b), string.sub(text, b + 2)
	for k,handler in pairs(SlashCmdList) do
		local i = 0
		while true do
			i = i + 1
			if not _G["SLASH_" .. k .. i] then
				break
			elseif _G["SLASH_" .. k .. i] == arg then
				handler(text)
				return
			end
		end
	end;
	print("No command found:", text)
end

local ChatFrameTemplate = {
	AddMessage = function(self, text)
		print((string.gsub(text, "|c%x%x%x%x%x%x%x%x(.-)|r", "%1")))
	end
}

for i=1,7 do
	local f = {}
	for k,v in pairs(ChatFrameTemplate) do
		f[k] = v
	end
	_G["ChatFrame"..i] = f
end
DEFAULT_CHAT_FRAME = ChatFrame1

function ChatFrame_AddMessageEventFilter()
end

debugstack = debug.traceback
date = os.date

function GetLocale()
	return "enUS"
end

function GetAddOnInfo()
	return
end

function GetNumAddOns()
	return 0
end

function getglobal(k)
	return _G[k]
end

function setglobal(k, v)
	_G[k] = v
end

local function _errorhandler(msg)
  print("--------- geterrorhandler error -------\n"..msg.."\n-----end error-----\n")
end

function geterrorhandler() 
  return _errorhandler
end

function InCombatLockdown()
	return false
end

function IsLoggedIn()
	return false
end

function GetFramerate()
	return 60
end

time = os.time

strmatch = string.match

function SendAddonMessage(prefix, message, distribution, target)
	assert(#prefix + #message < 255,
	       string.format("SendAddonMessage: message too long (%d bytes)",
			     #prefix + #message))
	-- CHAT_MSG_ADDON(prefix, message, distribution, sender)
        --print("SendMessage: "..prefix.." - "..distribution.."("..#message.."): "..message);
	WoWAPI_FireEvent("CHAT_MSG_ADDON", prefix, message, distribution, UnitName("player"))
end

function hooksecurefunc(func_name, post_hook_func)
	local orig_func = _G[func_name]

	_G[func_name] = function (...)
				local ret = { orig_func(...) }		-- yeahyeah wasteful, see if i care, it's a test framework
				post_hook_func(...)
				return unpack(ret)
			end
end

RED_FONT_COLOR_CODE = ""
GREEN_FONT_COLOR_CODE = ""

StaticPopupDialogs = {}

function WoWAPI_FireEvent(event,...)
	for frame, props in pairs(frames) do
		if props.events[event] then
			if props.scripts["OnEvent"] then
				for i=1,select('#',...) do
					_G["arg"..i] = select(i,...)
				end
				_G.event=event
				props.scripts["OnEvent"](frame,event,...)
			end
		end
	end
end

function WoWAPI_FireUpdate(forceNow)
	if forceNow then
		_time = forceNow
	end
	local now = GetTime()
	for frame,props in pairs(frames) do
		if props.isShow and props.scripts.OnUpdate then
			_G.arg1=now-props.timer
			props.scripts.OnUpdate(frame,now-props.timer)
			props.timer = now
		end
	end
end

local _metadata = {};
function SetAddOnMetadata(s,t,v)
  if(_metadata[s] == nil)
  then
    _metadata[s] = {};
  end
  _metadata[s][t] = v;
end

function GetAddOnMetadata(s,t)
  if(_metadata[s] == nil)
  then
    return nil;
  end
  return _metadata[s][t];
end

-- utility function for "dumping" a number of arguments (return a string representation of them)
function dump(...)
	local t = {}
	for i=1,select("#", ...) do
		local v = select(i, ...)
		if type(v)=="string" then
			tinsert(t, string.format("%q", v))
		else
			tinsert(t, tostring(v))
		end
	end
	return "<"..table.concat(t, "> <")..">"
end

function TEXT(...)
  return string.format(...);
end

function GetNumTalentTabs()
  return 0;
end

function IsInGuild()
  return 1;
end

function GetGuildInfo(unit)
  return GetCVar("GuildName");
end

function GetNumSavedInstances()
  return 1;
end

function GetNumSavedInstances()
  return 1;
end

function GetSavedInstanceInfo(index)
  if(index == 1)
  then
    return "Zul'Aman",164367,os.time() + 150;
  end
  return nil;
end

function DuplicateTable(tab)
  if(type(tab) ~= "table")
  then
    return nil;
  end
  local newtab = {};
  for n,v in pairs(tab)
  do
    if(type(v) == "table") -- recurs dup
    then
      newtab[n] = DuplicateTable(v);
    else
      newtab[n] = v;
    end
  end
  return newtab;
end

function PrintTable(tab,ofs)
  if(type(tab) ~= "table")
  then
    return nil;
  end
  if(ofs == nil) then ofs = ""; end
  for n,v in pairs(tab)
  do
    if(type(v) == "table") -- recurs dup
    then
      print(ofs..n.." (table):");
      PrintTable(v,ofs.." ");
    else
      print(ofs..n.." ("..type(v).."): "..tostring(v));
    end
  end
end

-- Wow global vars
UISpecialFrames = {};
SetCVar("realmName","Realm Name");
SetCVar("realmList","eu.logon");

-- Wow global strings
require("GlobalStrings");
