--[[
  GEM3_DataConverter by Kiki - Conseil des Ombres (Horde)
   GEM3 Data converter - For GEM3 Alpha/Beta testing
]]


--------------- Shared Constantes ---------------



--------------- Shared variables ---------------


--------------- Local Constantes ---------------

--------------- Local variables ---------------


--------------- Internal functions ---------------

local function _GDC_Convert124to125()
  GEM3_ChatMsg("GEM3_DataConverter: Conversion from 1.24 to 1.25 detected");
  for realm,realmdata in pairs(GEM3_Events)
  do
    GEM3_ChatMsg(" Converting realm '"..realm.."'");
    -- Convert event struct
    for ev_id,event in pairs(realmdata.events)
    do
      GEM3_ChatMsg("  Converting event '"..ev_id.."'");
      event.ev_duration = 0; -- Add ev_duration
      for name,player in pairs(event.players)
      do
        GEM3_ChatMsg("   Updating subscriber '"..name.."'");
        player.main_time = player.update_time; -- Add main_time
        player.alt_role = GEM3_ROLE_UNKNOWN; -- Add alt_role
      end
    end
    -- Convert my_deleted_events struct
    for ev_id,event in pairs(realmdata.my_deleted_events)
    do
      GEM3_ChatMsg("  Converting deleted events '"..ev_id.."'");
      event.ev_duration = 0; -- Add ev_duration
      for name,player in pairs(event.players)
      do
        GEM3_ChatMsg("   Updating subscriber '"..name.."'");
        player.main_time = player.update_time; -- Add main_time
        player.alt_role = GEM3_ROLE_UNKNOWN; -- Add alt_role
      end
    end
    -- Convert archived struct
    for ev_id,event in pairs(realmdata.archived)
    do
      GEM3_ChatMsg("  Converting archived events '"..ev_id.."'");
      event.ev_duration = 0; -- Add ev_duration
      for name,player in pairs(event.players)
      do
        GEM3_ChatMsg("   Updating subscriber '"..name.."'");
        player.main_time = player.update_time; -- Add main_time
        player.alt_role = GEM3_ROLE_UNKNOWN; -- Add alt_role
      end
    end
    -- Convert subscribed struct
    for ev_id,rerolls in pairs(realmdata.subscribed)
    do
      GEM3_ChatMsg("  Converting my subscriptions for event '"..ev_id.."'");
      for name,pl_sub in pairs(rerolls)
      do
        GEM3_ChatMsg("   Updating my subscription with '"..name.."'");
        pl_sub.main_time = pl_sub.update_time; -- Add main_time
        pl_sub.alt_role = GEM3_ROLE_UNKNOWN; -- Add alt_role
      end
    end
    -- Convert command struct
    for ev_id,cmdinfos in pairs(realmdata.commands)
    do
      GEM3_ChatMsg("  Converting commands for event '"..ev_id.."'");
      for name,pl_cmds in pairs(cmdinfos.cmds)
      do
        GEM3_ChatMsg("   Checking commands for '"..name.."'");
        for i,pl_cmd in ipairs(pl_cmds)
        do
          if(pl_cmd.cmd == GEM3_CMD_CMD_SUBSCRIBE)
          then
            GEM3_ChatMsg("    Updating command Subscribe at "..pl_cmd.stamp);
            tinsert(pl_cmd.params,GEM3_ROLE_UNKNOWN); -- Add alt_role
            tinsert(pl_cmd.params,pl_cmd.stamp); -- Add main_time
            tinsert(pl_cmd.params,pl_cmd.stamp); -- Add update_time
          else
            GEM3_ChatMsg("    Ignoring command '"..pl_cmd.cmd.."'");
          end
        end
      end
    end
  end
  GEM3_ChatMsg("GEM3_DataConverter: Conversion completed!");
  GEM3_Config.min_config_version = 1.25;
end

local function _GDC_Convert125to126()
  GEM3_ChatMsg("GEM3_DataConverter: Conversion from 1.25 to 1.26 detected");
  GEM3_Config.locale = GetLocale();

  local search_str;
  local repl_str = "guild%-";
  if(GEM3_Config.locale == "frFR")
  then
    GEM3_ChatMsg("GEM3_DataConverter: Detecting frFR locale, converting GuildChannel name");
    search_str = "guilde%-";
  end

  if(search_str) -- Convert config
  then
    for realm,realmdata in pairs(GEM3_Config)
    do
      if(type(realmdata) == "table") -- Realm infos
      then
        for name,infos in pairs(realmdata)
        do
          for i,chan in ipairs(infos.channels)
          do
            GEM3_ChatMsg("GEM3_DataConverter: Converting Config ChannelName '"..chan.name.."' for player "..name.." from realm "..realm);
            chan.name = string.gsub(chan.name,search_str,repl_str);
          end
        end
      end
    end
  end

  for realm,realmdata in pairs(GEM3_Players)
  do
    for name,chandata in pairs(realmdata)
    do
      GEM3_ChatMsg("GEM3_DataConverter: Converting Players ChannelName '"..name.."' from realm "..realm);
      if(search_str) -- Convert players
      then
        local new_name = string.gsub(name,search_str,repl_str);
        if(new_name ~= name)
        then
          realmdata[new_name] = chandata;
          realmdata[name] = nil;
        end
      end
      for player,pltab in pairs(chandata)
      do
        pltab.tags = {};
      end
    end
  end

  for realm,realmdata in pairs(GEM3_Events)
  do
    GEM3_ChatMsg(" Converting realm '"..realm.."'");
    -- Convert event struct
    for ev_id,event in pairs(realmdata.events)
    do
      GEM3_ChatMsg("  Converting event '"..ev_id.."'");
      event.ev_subtype = 0; -- Add ev_subtype
      if(search_str) -- Convert config
      then
        GEM3_ChatMsg("GEM3_DataConverter: Converting EventChannel '"..event.channel.."' for EventID "..ev_id);
        event.channel = string.gsub(event.channel,search_str,repl_str);
      end
    end
    -- Convert my_deleted_events struct
    for ev_id,event in pairs(realmdata.my_deleted_events)
    do
      GEM3_ChatMsg("  Converting deleted event '"..ev_id.."'");
      event.ev_subtype = 0; -- Add ev_subtype
      if(search_str) -- Convert config
      then
        GEM3_ChatMsg("GEM3_DataConverter: Converting EventChannel '"..event.channel.."' for EventID "..ev_id);
        event.channel = string.gsub(event.channel,search_str,repl_str);
      end
    end
    -- Convert archived struct
    for ev_id,event in pairs(realmdata.archived)
    do
      GEM3_ChatMsg("  Converting archived event '"..ev_id.."'");
      event.ev_subtype = 0; -- Add ev_subtype
      if(search_str) -- Convert config
      then
        GEM3_ChatMsg("GEM3_DataConverter: Converting EventChannel '"..event.channel.."' for EventID "..ev_id);
        event.channel = string.gsub(event.channel,search_str,repl_str);
      end
    end
    if(search_str) -- Convert config
    then
      -- Convert command struct
      for ev_id,cmdinfos in pairs(realmdata.commands)
      do
        GEM3_ChatMsg("  Converting channel commands for event '"..ev_id.."'");
        cmdinfos.channel = string.gsub(cmdinfos.channel,search_str,repl_str);
        for name,pl_cmds in pairs(cmdinfos.cmds)
        do
          GEM3_ChatMsg("   Checking commands for '"..name.."'");
          for i,pl_cmd in ipairs(pl_cmds)
          do
            GEM3_ChatMsg("    Updating Channel for command "..pl_cmd.cmd);
            pl_cmd.channel = string.gsub(pl_cmd.channel,search_str,repl_str);
          end
        end
      end
    end
  end
  GEM3_ChatMsg("GEM3_DataConverter: Conversion completed!");
  GEM3_Config.min_config_version = 1.26;
end

local function _GDC_Convert126to127()
  GEM3_ChatMsg("GEM3_DataConverter: Conversion from 1.26 to 1.27 detected");
  GEM3_Config.locale = GetLocale();

  for realm,realmdata in pairs(GEM3_Events)
  do
    GEM3_ChatMsg(" Converting realm '"..realm.."'");
    -- Add others_deleted_events
    realmdata.others_deleted_events = {};
  end
  GEM3_ChatMsg("GEM3_DataConverter: Conversion completed!");
  GEM3_Config.min_config_version = 1.27;
end

--------------- Event function ---------------

function GEM3_DataConverter_OnEvent(self,event,...)
  if(event == "VARIABLES_LOADED")
  then
    if(GEM3_MIN_CONFIG_VERSION_RESET >= 1.25)
    then
      if(GEM3_Config and GEM3_Config.min_config_version and GEM3_Config.min_config_version == 1.24)
      then
        _GDC_Convert124to125();
      end
    end
    if(GEM3_MIN_CONFIG_VERSION_RESET >= 1.26)
    then
      if(GEM3_Config and GEM3_Config.min_config_version and GEM3_Config.min_config_version == 1.25)
      then
        _GDC_Convert125to126();
      end
    end
    if(GEM3_MIN_CONFIG_VERSION_RESET >= 1.27)
    then
      if(GEM3_Config and GEM3_Config.min_config_version and GEM3_Config.min_config_version == 1.26)
      then
        _GDC_Convert126to127();
      end
    end
  end
end


--------------- GEM3_DataConverter Main Frame ---------------

--Event Driver
if(not GEM3_DataConverterFrame)
then
  CreateFrame("Frame", "GEM3_DataConverterFrame");
end
GEM3_DataConverterFrame:Hide();
--Event Registration
GEM3_DataConverterFrame:RegisterEvent("VARIABLES_LOADED");
--Frame Scripts
GEM3_DataConverterFrame:SetScript("OnEvent", GEM3_DataConverter_OnEvent);

